PPUCTRL = $2000
PPUMASK = $2001
PPUSTATUS = $2002
PPUADDR = $2006
PPUDATA = $2007
last_PPUCTRL = $0058
last_chrbank = $0059

ciSrc = $C0
ciSrcBank = $C2
ciBlocksLeft = $C3
ciDst = $C4
ciBufStart = $C6
ciBufEnd = $C7
PB53_outbuf = $0100
identity = $C2C0  ; used for bank switching

.segment "INESHDR"
  .byte "NES",$1A
  .byte 4  ; PRG ROM size in 16384 byte units
  .byte 0  ; CHR ROM size in 8192 byte units (0 means 8192 byte RAM)
  .byte $21  ; lower nibble of mapper (UNROM = $02), horz mirroring
  .byte $00  ; upper nibble of mapper number

; MAIN PRG ROM PATCH ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

INES_HEADER_SIZE = 16
.macro copy_till copyend
.local here, copyend_, fileoffset
here = * - bank2start + $8000
copyend_ = copyend
fileoffset = * - bank2start + INES_HEADER_SIZE
  .out .sprintf("here:$%04X end:$%04X", here, copyend_)
  .if copyend_ - here > 0
    .incbin "SolarWars2001.NES", fileoffset, copyend_ - here
  .endif
.endmacro

.macro pad_till padend, padvalue
.local here, padend_
here = * - bank2start + $8000
padend_ = padend
  .if padend_ - here > 0
    .res padend_ - here, padvalue
  .endif
.endmacro

.segment "ROM23"

bank2start:
  copy_till $C30B
entry_c30b:
reset:
  sei
  ldx #$00
  stx entry_c30b+2
  jmp init_machine
  
;;
; Load bank X (where X = 0-3) into CHR RAM
loadbank:
  lda #0
  sta PPUCTRL  ; disable nmi
  sta loadbank+1
  jmp loadbank_0
loadbank_rtl:
  lda #2
  sta loadbank_rtl+1
  bit PPUSTATUS
  lda $58
  sta PPUCTRL
  rts

  pad_till $C343, $FF
  .include "loadbank_patches.s"

  ; 43110 found and fixed a bug that was causing a random freeze
  ; during initial placement of the players' tanks: "it basically
  ; corrects the InitP1X loop point in the subroutine InitTanks"
  ; http://forums.nesdev.com/viewtopic.php?p=136607#p136607
  copy_till $F566
  jmp $f552
  
  copy_till $FFE0
  pad_till $FFEB, $FF
reset_ffeb:
  lda #$81
  sta $5000
  sei
ldxinstr:
  ldx #$FF
  txs
  stx ldxinstr+1
  jmp ($FFFC)
  copy_till $10000

; INIT CODE AND BANK 1-3 LOADER ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.segment "ROM0"

unpb53_some:
  .include "unpb53.s"

loadbank_0:
  ; Load the address
  lda #0
  sta PPUADDR
  sta PPUADDR
  sta PPUMASK
  lda chrstart_lo,x
  sta ciSrc
  lda chrstart_hi,x
  sta ciSrc+1
  lda chrstart_bank,x
  sta ciSrcBank
  ldx #8192 / 128
  stx ciBlocksLeft
eightmoretiles:
  lda #$80
  sta ciBufEnd
  asl a
  sta ciBufStart
  jsr unpb53_some
  ldx #0
copyloop:
  lda PB53_outbuf,x
  sta PPUDATA
  inx
  bpl copyloop
  dec ciBlocksLeft
  bne eightmoretiles

  jmp loadbank_rtl

.proc init_machine
  stx $2000
  stx $2001
  stx $4015
  txa
  dex
  txs
  bit PPUSTATUS
@ppuwarmup1:
  bit PPUSTATUS
  bpl @ppuwarmup1
  cld

  ; Clear all RAM (to ensure C-like starting conditions)
  ldy #$07
  sty $01
  tay
  sty $00
:
  sta ($00),y
  iny
  bne :-
  dec $01
  bpl :-

@ppuwarmup2:
  bit PPUSTATUS
  bpl @ppuwarmup2

  ; Do everything up till the first CHR ROM bank switch
  jsr $E119
  jsr $E13F
  jmp $C349
.endproc

chrstart_lo:
  .byte <chrbank0_bin, <chrbank1_bin, <chrbank2_bin, <chrbank3_bin
chrstart_hi:
  .byte >chrbank0_bin, >chrbank1_bin, >chrbank2_bin, >chrbank3_bin
chrstart_bank:
  .byte 1, 0, 0, 0

chrbank1_bin:
  .incbin "chrbank1.bin"
chrbank2_bin:
  .incbin "chrbank2.bin"
chrbank3_bin:
  .incbin "chrbank3.bin"

; BANK 0 LOADER

.segment "ROM1"

unpb53_some_1:
  .include "unpb53.s"

.assert unpb53_some_1 = unpb53_some, error, "decompressor is at different locations in banks 0 and 1"

chrbank0_bin:
  .incbin "chrbank0.bin"

